//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/SampleDesigner/RealSpacePanel.cpp
//! @brief     Implements class RealSpacePanel
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/SampleDesigner/RealSpacePanel.h"
#include "GUI/View/Common/StyledToolBar.h"
#include "GUI/View/Realspace/RealSpaceCanvas.h"
#include <QVBoxLayout>

RealSpacePanel::RealSpacePanel(SampleModel* sampleModel, QItemSelectionModel* selectionModel,
                               QWidget* parent)
    : QWidget(parent)
    , m_canvas(new RealSpaceCanvas)
    , m_sampleModel(sampleModel)
    , m_selectionModel(selectionModel)
{
    setWindowTitle("Real Space");
    setObjectName("Sample3DPanel");

    auto* toolbar = new StyledToolBar(this);

    const auto createAction = [&](const QString& text, const QString& tooltip) -> QAction* {
        auto* action = new QAction(text, this);
        action->setToolTip("Save 3D real space view as .png file");
        toolbar->addAction(action);
        toolbar->addSeparator();
        return action;
    };

    auto* action = createAction("Save Picture", "Save 3D real space view as .png file");
    connect(action, &QAction::triggered, m_canvas, &RealSpaceCanvas::onSavePictureAction);

    action = createAction("Default View", "Reset view and zoom level to default");
    connect(action, &QAction::triggered, m_canvas, &RealSpaceCanvas::onDefaultViewAction);

    action = createAction("Side View", "View sample from the side at current zoom level");
    connect(action, &QAction::triggered, m_canvas, &RealSpaceCanvas::onSideViewAction);

    action = createAction("Top View", "View sample from the top at current zoom level");
    connect(action, &QAction::triggered, m_canvas, &RealSpaceCanvas::onTopViewAction);

    action = createAction("Enlarge", "Increase layer size");
    connect(action, &QAction::triggered, [this]() { m_canvas->onChangeLayerSizeAction(1.25); });

    action = createAction("Reduce", "Decrease layer size");
    connect(action, &QAction::triggered, [this]() { m_canvas->onChangeLayerSizeAction(0.8); });

    action = createAction("Lock View", "Lock/unlock current sample selection");
    action->setCheckable(true);
    connect(action, &QAction::triggered, m_canvas, &RealSpaceCanvas::onLockViewAction);

    auto* mainLayout = new QVBoxLayout(this);
    mainLayout->setMargin(0);
    mainLayout->setSpacing(0);
    mainLayout->setContentsMargins(0, 0, 0, 0);
    mainLayout->addWidget(toolbar);
    mainLayout->addWidget(m_canvas);
}

QSize RealSpacePanel::sizeHint() const
{
    return QSize(300, 300);
}

void RealSpacePanel::showEvent(QShowEvent*)
{
    m_canvas->setModel(m_sampleModel, m_selectionModel);
}

void RealSpacePanel::hideEvent(QHideEvent*)
{
    m_canvas->setModel(nullptr, nullptr);
}
